#include <stdio.h>
#include <stdlib.h>
#include <io.h>
#include <fcntl.h>

typedef struct {
  unsigned wSignature;
  unsigned wPartPage;
  unsigned wPageCnt;
  unsigned wReloCnt;
  unsigned wHdrSize;
  unsigned wMinAlloc;
  unsigned wMaxAlloc;
  unsigned wInitSS;
  unsigned wInitSP;
  unsigned wChkSum;
  unsigned wInitIP;
  unsigned wInitCS;
  unsigned wTablOff;
  unsigned wOverlayNo;
} MZHeader;

enum {
  MZ_SIGNATURE = 0x5a4d,
  PAGE_SIZE = 0x200
};

int main(int argc, char** argv)
{
  int file;
  int image_len;
  MZHeader header;

  const char* filename = argv[1];

  if(argc<2)
  {
    printf("Specify the name of EXE file to look into\n");
    return 1;
  }

  file = open(filename, O_RDONLY|O_BINARY);
  if(file<0)
  {
    printf("Can't open file %s\n", filename);
    return 1;
  }
  read(file, &header, sizeof header);
  close(file);

  if(header.wSignature != MZ_SIGNATURE)
  {
	printf("%s is not a DOS MZ executable\n", filename);
    return 1;
  }

  /* first get code size */
  image_len  = header.wPageCnt << 5;
  image_len -= (PAGE_SIZE-header.wPartPage) >> 4;
  image_len -= header.wHdrSize;

  /* add the stack size */
  image_len += (header.wInitSP+15)>>4;

  /* add minimum required memory past the code image */
  image_len += header.wMinAlloc;

  printf("The EXE requires %d paragraphs (%d KB) of conventional memory\n", image_len, (image_len+63)>>6);

  return 0;
}
